@@ -631,7 +631,7 @@ peas_gtk_plugin_manager_view_query_tooltip (GtkWidget  *widget,
     return FALSE;
 
   /* Avoid having markup in a translated string */
-  to_bold = g_strdup_printf (_("The plugin '%s' could not be loaded"),
+  to_bold = g_strdup_printf (_("The plugin “%s” could not be loaded"),
                              peas_plugin_info_get_name (info));
 
   /* Keep separate because some translations do special things
@@ -651,7 +651,6 @@ peas_gtk_plugin_manager_view_query_tooltip (GtkWidget  *widget,
   return TRUE;
 }
 
-
 static void
 peas_gtk_plugin_manager_view_row_activated (GtkTreeView       *tree_view,
                                             GtkTreePath       *path,
@@ -660,6 +659,7 @@ peas_gtk_plugin_manager_view_row_activated (GtkTreeView       *tree_view,
   PeasGtkPluginManagerView *view = PEAS_GTK_PLUGIN_MANAGER_VIEW (tree_view);
   PeasGtkPluginManagerViewPrivate *priv = GET_PRIV (view);
   GtkTreeIter iter;
+  GtkTreeViewClass *tree_view_class;
 
   if (!gtk_tree_model_get_iter (gtk_tree_view_get_model (tree_view), &iter, path))
     return;
@@ -668,6 +668,10 @@ peas_gtk_plugin_manager_view_row_activated (GtkTreeView       *tree_view,
 
   if (peas_gtk_plugin_manager_store_can_enable (priv->store, &iter))
     toggle_enabled (view, &iter);
+
+  tree_view_class = GTK_TREE_VIEW_CLASS (peas_gtk_plugin_manager_view_parent_class);
+  if (tree_view_class->row_activated != NULL)
+    tree_view_class->row_activated (tree_view, path, column);
 }
 
 static void
@@ -750,8 +754,7 @@ peas_gtk_plugin_manager_view_dispose (GObject *object)
   PeasGtkPluginManagerView *view = PEAS_GTK_PLUGIN_MANAGER_VIEW (object);
   PeasGtkPluginManagerViewPrivate *priv = GET_PRIV (view);
 
-  g_clear_pointer (&priv->popup_menu,
-                   (GDestroyNotify) gtk_widget_destroy);
+  g_clear_pointer (&priv->popup_menu, gtk_widget_destroy);
 
   g_clear_object (&priv->engine);
   g_clear_object (&priv->store);
@@ -818,7 +821,7 @@ peas_gtk_plugin_manager_view_class_init (PeasGtkPluginManagerViewClass *klass)
    * connect to this signal and add your menuitems to the @menu.
    */
   signals[POPULATE_POPUP] =
-    g_signal_new ("populate-popup",
+    g_signal_new (I_("populate-popup"),
                   the_type,
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (PeasGtkPluginManagerViewClass, populate_popup),
